#version 330 core

// Ouput data
layout(location = 0) out vec4 out_color;

uniform sampler2D depth_texture;
uniform sampler2D scene_texture;
uniform sampler2D blur_texture;
uniform sampler2D scene_texture_small;
uniform int state;

//the size of the textures
uniform int swidth;
uniform int sheight;
uniform int dwidth;
uniform int dheight;

//if true then active near/far-blur
uniform	bool near_on;
uniform	bool far_on;

//radius of CoC(circle of Confusion) for near- and far-blurring
uniform	float near_r;
uniform	float far_r;

//start of the end of near-blur
uniform	float near_start;

//end of the end of near-blur(after that there is no blur till far-blur)
uniform	float near_end;

//start of the end of the unblurred part
uniform	float far_start;

//full far-blur from here
uniform	float far_end;

//scale from high- to low-res-coc
uniform float radiusScale;

//shows near-far-coc in color
uniform float Dof_show;

uniform float poisson[16];

//in vec2 UV;
in vec2 VS_texCoords;

float ComputeDepthBlurRadius(float depth)
{
	float f;

	if(depth<near_end)
	{
		if(depth<near_start)
		{
			f=near_r;
		}
		else
		{
			f=near_r*((near_end-depth)/(near_end-near_start));
		}
	}
	else
	{
		if(depth>far_start)
		{
			if(depth>=far_end)
			{
				f=far_r;
			}
			else
			{
				f=far_r*((depth-far_start)/(far_end-far_start));
			}
		}
		else
		{
			f=0;
		}
	}
	return f;
}

//as seen in Scheuermann 2004
vec4 DofFilter()
{
	int NUM_TAPS=16;
	
	vec4 cOut;
	cOut=texture2D(depth_texture, VS_texCoords);
	float Depth=cOut.x;

	cOut=vec4(0,0,0,0);

	float discRadius=ComputeDepthBlurRadius(Depth);
	
	float rScale=0.6f;

	//float discRadius, discRadiusLow, centerDepth;
	float discRadiusLow, centerDepth;

	cOut=texture2D(depth_texture, VS_texCoords);
	centerDepth=cOut.x;//cOut.a

	//convert the depth intur the blur radius for the pixels
	//discRadius=abs(centerDepth*vMaxCoc.y-vMaxCoc.x);
	discRadiusLow=discRadius*rScale;//radius for low resolution

	//re-use
	cOut=vec4(0,0,0,0);

	vec2 pixelSizeLow=vec2(1/swidth,1/sheight)/4;
	vec2 pixelSizeHigh=vec2(1/swidth,1/sheight);

	for(int j=0;j<NUM_TAPS;j++)
	{
		//compute tap texture coordinates
		vec2 coordLow=VS_texCoords+(pixelSizeLow*poisson[j]*discRadiusLow);
		vec2 coordHigh=VS_texCoords+(pixelSizeHigh*poisson[j]*discRadius);

		//fetch high-res tap
		vec4 tapLow=texture2D(scene_texture_small,coordLow);
		vec4 tapHigh=texture2D(scene_texture,coordHigh);

		vec4 tapLow2=texture2D(depth_texture,coordHigh);
		vec4 tapHigh2=texture2D(depth_texture,coordHigh);

		tapLow.a=tapLow2.x;
		tapHigh.a=tapHigh2.x;

		//merge low- and high-resolution
		float tapBlur=tapHigh.a;//abs(tapHigh.a*2-1);
		vec4 tap=mix(tapHigh,tapLow,tapBlur);

		//smart blur ignores taps that are closer than the center tap and in focus
		tap.a=(tap.a>=centerDepth)?1.0:tap.a;//abs(tap.a*2-1);

		cOut.rgb+=tap.rgb*tap.a; //accumulate
		cOut.a+=tap.a;
	}
	return(cOut/cOut.a);
}

void doHBlur()
{
	vec4 color = texture2D(scene_texture, VS_texCoords);
	
	float step = 1/swidth;
	step=1;
	//color *= 6;
	color += texture2D(scene_texture, VS_texCoords+vec2(-2*step, 0.0));
	color += texture2D(scene_texture, VS_texCoords+vec2(-1*step, 0.0));
	color += texture2D(scene_texture, VS_texCoords+vec2( 1*step, 0.0));
	color += texture2D(scene_texture, VS_texCoords+vec2( 2*step, 0.0));
		
	out_color = color/16;
}

void doVBlur()
{
	vec4 color = texture2D(blur_texture, VS_texCoords);

	//vertical blur
	float step = 1/sheight;
	step=1;
	color *= 6;
	color += texture2D(blur_texture, VS_texCoords+vec2(0.0, -2*step));
	color += texture2D(blur_texture, VS_texCoords+vec2(0.0, -1*step))*4;
	color += texture2D(blur_texture, VS_texCoords+vec2(0.0,  1*step))*4;
	color += texture2D(blur_texture, VS_texCoords+vec2(0.0,  2*step));
		
	out_color = texture2D(scene_texture, VS_texCoords) + color/16;
}

void main(){
	if(state==0)
	{
		doHBlur();
	}
	else
	{
		if(state==1)
		{
			doVBlur();
		}
		else
		{
			if(state==2)
			{
				out_color=texture2D(scene_texture,VS_texCoords);
			}
			else
			{
				if(state==3)
				{
					vec4 col=texture2D(depth_texture, VS_texCoords);
					float depth=col.x;
					if((depth>=far_start)||(depth<=near_end))
					{
						if(Dof_show==0)
						{
							out_color=DofFilter();
						}
						else
						{
						//color CoC-area for debugging
							if(Dof_show==1)
							{	
								//color near-coc
								if(depth<=near_end)
								{
									out_color=vec4(0,0,1,0);
								}
								else
								{
									out_color=DofFilter();
								}
							}
							else
							{
								if(Dof_show==2)
								{
									//color far-coc
									if(depth>=far_start)
									{
										out_color=vec4(1,0,0,0);
									} 
									else
									{
										out_color=DofFilter();
									}
								}
								else
								{
									out_color=vec4(0,1,0,0);
								}
							}
						}
					}
					else
					{
						out_color=texture2D(scene_texture, VS_texCoords);
					}
				}
			}
		}
	}
}